﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Settlement.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Settlement',
    xtype: 'App.View.Kivii.Finances.Settlement.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Settlement/QueryEx.json',
    requires: ['App.Control.DownloadField','System.Control.CategoryField'],
    title: '结算管理',
    iconCls: 'x-fa fa-newspaper-o',
    filter: {
        IsPayed: false,
        OffsetKvid: '00000000-0000-0000-0000-000000000000',
        OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400,
        groupField: 'PayerName',
        sorters: [{
            property: 'OwnerName',
            direction: 'DESC'
        }, {
            property: 'PayerName',
            direction: 'DESC'
        }, {
            property: 'BizId',
            direction: 'ASC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'groupingsummary',
        groupHeaderTpl: [
            '<div>{name:this.formatName} ({rows.length} 条)</div>',
            {
                formatName: function(name) {
                    return name;
                }
            }
        ]
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '当前页 结算金额: {amount} &nbsp;&nbsp;已收款: {amountPayed} &nbsp;&nbsp;未收款: {amountUnpayed}'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        },{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'combo',
                queryMode: 'local',
                width:150,
                displayField: 'displayInfo',
                valueField: 'isPayed',
                value: false,
                editable: false,
                filterPickList: true,
                reference: 'isPayed',
                store: {
                    fields: ['displayInfo', 'isPayed'],
                    data: [{
                        displayInfo: '全部',
                        isPayed: null
                    }, {
                        displayInfo: '已收款',
                        isPayed: true
                    }, {
                        displayInfo: '未收款',
                        isPayed: false
                    }]
                },
                listeners: {
                    select: '_onSelectIsPayed'
                }
            }, {
                xtype: 'categoryField',
                emptyText:'类型',
                width:150,
                entityType: 'Kivii.Finances.Entities.Settlement',
                internalCode: 'Type',
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanType'
                    }
                },
                listeners: {
                    select: '_selectType'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '业务日期  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'PayerName,BizId,OperatorName',
                emptyText: '模糊搜索付款单位、所属业务或操作人',
            }, {
                xtype: 'checkbox',
                // value:true,
                labelWidth: 100,
                fieldLabel: '仅显示未有账单',
                iconAlign: 'left',
                handler: '_onCheck'
            }, {
                text: '新增结算',
                iconCls: 'x-fa fa-plus',
                handler: '_onCreateSettlement'
            }, {
                text: '生成账单',
                iconCls: 'x-fa fa-file-text-o',
                handler: '_onGenerateBill',
                bind: {
                    disabled: '{!_selection}'
                }
            }, {
                text: '收款登记',
                iconCls: 'x-fa fa-tags',
                handler: '_onPaying',
                bind: {
                    disabled: '{!_selection}'
                }
            }, {
                text: '撤销',
                iconCls: 'x-fa fa-trash',
                iconAlign: 'left',
                handler: '_onOffsetEntity',
                bind: {
                    disabled: '{!_selection}'
                }
            }, {
                xtype: 'downloadField',
                dataSource: '/Restful/Kivii.Finances.Entities.Settlement/QueryEx.xlsx', //数据源
                fileName: '数据导出',
                window: {
                    type: 'App.Control.Kivii.Finances.General.Template.Selector',
                    title: '选择模板',
                    iconCls: 'x-fa fa-file-excel-o',
                    autoScroll: true,
                    width: 700,
                    height: 350,
                    closeAction: 'hide',
                    modal: true,
                    layout: 'fit',
                    filter: {
                        FolderPath: '/Finances/Settlement/Files',
                        TagsContains: 'Query'
                    }
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'BizId',
            text: '所属业务',
            width: 170,
            // sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'OwnerName',
            text: '所属部门',
            width: 120
        }, {
            dataIndex: 'OperateTime',
            text: '业务日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'AmountPlan',
            align: 'right',
            text: '原金额',
            width: 100,
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '结算金额',
            width: 100,
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已付金额',
            width: 100,
            renderer: '_renderAmount',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'CreateTime',
            text: '结算日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'PayedTime',
            text: '付款日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'Metadata.TOTALPRICE',
            align: 'right',
            text: '报告金额',
            width: 100
        }, {
            dataIndex: 'Type',
            text: '类型',
            width: 90,
            renderer: '_renderType'
        }, {
            dataIndex: 'Category',
            text: '分类',
            width: 90
        }, {
            dataIndex: 'Metadata.PARENT',
            text: '原账单',
            width: 120
        }, {
            dataIndex: 'OperatorName',
            text: '操作人',
            width: 90
        }, {
            xtype: 'actioncolumn',
            text: '操作&nbsp;&nbsp;&nbsp;&nbsp;附件',
            align: 'center',
            items: [{
                text: '查看',
                iconCls: 'x-fa fa-search',
                handler: '_onReadSettlement'
            }, '->', {
                text: '附件',
                iconCls: 'x-fa fa-file-text-o',
                handler: '_onAttachment'
            }],
            width: 120
        }, {
            dataIndex: 'Summary',
            text: '摘要',
            width: 200
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _viewReady: function() {
            let _me = this;
            let _view = _me.getView();
            let _vm = _me.getViewModel();
            let _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
                let _rtns = Ext.decode(_operation.getResponse().responseText);
                let _results = _rtns.Results;
                let _amount = 0;
                let _amountPayed = 0;
                let _amountUnpayed = 0;
                for (let _i = 0; _i < _results.length; _i++) {
                    _amount += _results[_i].Amount;
                    _amountPayed+=_results[_i].AmountPayment;
                }
                _amountUnpayed=parseFloat(_amount-_amountPayed);
                _vm.set('amount', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_amount.toFixed(2)));
                _vm.set('amountPayed', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_amountPayed.toFixed(2)));
                _vm.set('amountUnpayed', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_amountUnpayed.toFixed(2)));
            });
        },
        
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Report") {
                return "检测费";
            }
            if (_value === "Certificate") {
                return "证书费";
            }
            if (_value === "Tag") {
                return "挂签费";
            }
            if (_value === "Authentication") {
                return "认证费";
            }
            if (_value === "SoftwareSupervision") {
                return "软件监理费";
            }
            if (_value === "SoftwareCostEvaluation") {
                return "软件造价评估费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Train") {
                return "培训费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            } else _meta.style = 'background-color:#F0E68C';
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== null) _meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        
        _cleanType: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.Type;
            _view.reload();
        },
        _selectType: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            if (_view.filter === null) _view.setFilter({});
            _view.filter.Type = _record.data.InternalCode;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        },
        _onSelectIsPayed: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.IsPayed = _record.data.isPayed;
            _view.reload();
        },
        _onCheck: function(_ctrl, _isChecked) {
            let _controller = this;
            let _view = _controller.getView();
            if (!_isChecked) delete _view.filter.IsBilled;
            else _view.filter.IsBilled = !_isChecked;
            _view.reload();
        },
        _onCreateSettlement: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                header: false
            });
            let _item= {
                Metadata: {},
                Kvid: null,
                OffsetKvid: null,
                BizId: new Date().getTime(),
                BizType: 'Kivii.Finances.Entities.Settlement',
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            };
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1300,
                height: 720,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_item);
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onReadSettlement: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 680,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data);
            _form.on('entityDeleted', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
            _form.on('entityUpdated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onAttachment: function(_ctrl, _rowIndex, _colIndex, _item, _e, _record, _row) {
            var _controller = this;
            var _view = _controller.getView();
            var _form = Ext.create('App.Control.Kivii.Finances.General.Storage.List', {
                header: false,
                commands: {
                    FolderPath: '/Finances/Settlement/Files',
                    FolderType: 'Kivii.Finances.Entities.Settlement',
                    OwnerKvid: _record.data.Kvid
                }
            });
            var _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _item.text, //'图片集合',
                iconCls: _item.iconCls, //'x-fa fa-picture-o',
                // header:false,
                iconAlign: 'left',
                autoScroll: true,
                width: 850,
                height: 500,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.setGlobalFilter({
                OwnerKvid: _record.data.Kvid
            });
            _form.setFilter({});
            _form.reload();
        },
        _onOffsetEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (_selecteds.length <= 0) return;
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm(_ctrl.text, "您确认要处理当前选中数据？", _controller.beforeDeleteEntity, _controller);
        },
        beforeDeleteEntity: function(_optional) {
            if (_optional != "yes") return;
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view=_controller.getView();
            let _selecteds = _vm.data.selections || [];
            if (_selecteds.length <= 0) return;
            let _entityType = '';
            let _offsets = [];
            let _deletes = [];
            for (let _i = 0; _i < _selecteds.length; _i++) {
                let _item = _selecteds[_i].data;
                _entityType = _item.BizType;
                if (_item.AmountPayment === 0) _deletes.push(_item.Kvid);
                else if (_item.AmountPayment > 0) _offsets.push(_item.Kvid);
            }
            let _jsonData = {};
            _jsonData.EntityType = _entityType;
            _jsonData.Kvids = [];
            if (_offsets.length > 0) {
                _jsonData.Kvids = _offsets;
                Ext.Ajax.request({
                    url: '/Restful/Kivii.Finances.Entities.Settlement/Offset.json',
                    jsonData: _jsonData,
                    success: function(_response) {
                        let _rtns = Ext.decode(_response.responseText);
                        Ext.toast('删除成功!');
                        _view.reload();
                    },
                    failure: function(_response) {
                        Ext.toast('删除失败!');
                    }
                });
            }
            if (_deletes.length > 0) {
                _jsonData.Kvids = _deletes;
                Ext.Ajax.request({
                    url: '/Restful/Kivii.Finances.Entities.Settlement/Delete.json',
                    jsonData: _jsonData,
                    success: function(_response) {
                        let _rtns = Ext.decode(_response.responseText);
                        Ext.toast('删除成功!');
                        _view.reload();
                    },
                    failure: function(_response) {
                        Ext.toast('删除失败!');
                    }
                });
            }

        },
        _onGenerateBill: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;

            let _bill = {
                Metadata: {},
                Kvid: null,
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            };
            for (let _i = 0; _i < _selecteds.length; _i++) {
                _bill.PayerName = _selecteds[_i].data.PayerName;
                _bill.PayerKvid = _selecteds[_i].data.PayerKvid;
                _bill.Details.push(_selecteds[_i].data);
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Bill.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 700,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_bill);
            _win.show();
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onPaying: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) {
                Ext.toast("请选中需收款登记的结算信息!");
                return;
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Paying.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 500,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_selecteds);
            _win.show();
            _form.on('entityPaying', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        }
    }
});